@@ -8,17 +8,16 @@ import re
 
 import streamlit as st
 
-import PIL.Image as Image
 import streamlit.components.v1 as components
 
 from datasetinsights.datasets.unity_perception import AnnotationDefinitions, MetricDefinitions
 from datasetinsights.datasets.unity_perception.captures import Captures
 
-import visualization.visualizers as v
 import helpers.custom_components_setup as cc
+from datasetvisualizer.Dataset import Dataset
 
 
-def datamaker_dataset(path: str) -> Optional[Dict[int, Tuple[any, any, any, int, any]]]:
+def datamaker_dataset(path: str) -> Optional[Dict[int, Dataset]]:
     """ Reads the given path as a datamaker dataset
 
         Assumes that the given path contains a folder structure as follows:
@@ -44,10 +43,9 @@ def datamaker_dataset(path: str) -> Optional[Dict[int, Tuple[any, any, any, int,
                     instance_num = int(instance[instance.rfind("instance_") + len("instance_"):])
                     for attempt in [h.path for h in os.scandir(instance) if h.is_dir()]:
                         if re.match(".*attempt_[0-9]*", attempt):
-                            ann_def, metric_def, cap = load_perception_dataset(attempt)
-                            if ann_def is not None:
-                                instances[instance_num] = ann_def, metric_def, cap, len(
-                                    cap.captures.to_dict('records')), attempt
+                            ds = Dataset(attempt)
+                            if ds.dataset_valid:
+                                instances[instance_num] = ds
     except PermissionError:
         return None
 
@@ -57,26 +55,6 @@ def datamaker_dataset(path: str) -> Optional[Dict[int, Tuple[any, any, any, int,
         return None
 
 
-@st.cache(show_spinner=True, allow_output_mutation=True)
-def load_perception_dataset(path: str) -> Tuple[AnnotationDefinitions, MetricDefinitions, Captures]:
-    """ Reads the given path as a normal Perception dataset and returns Dataset Insight objects that represent it
-
-    :param path: root directory for a normal Perception dataset
-    :type path: str
-
-    :return: Annotations, Metrics and Captures for the given perception_dataset, if the dataset is invalid it returns
-            (None, None, None)
-    :rtype: (AnnotationDefinitions, MetricDefinitions, Captures)
-    """
-    try:
-        ann_def = AnnotationDefinitions(path)
-        metric_def = MetricDefinitions(path)
-        cap = Captures(path)
-        return ann_def, metric_def, cap
-    except Exception:
-        return None, None, None
-
-
 def create_session_state_data(attribute_values: Dict[str, any]):
     """ Takes a dictionary of attributes to values to create the streamlit session_state object. 
     The values are the default values
@@ -153,17 +131,6 @@ def create_sidebar_labeler_menu(available_labelers: List[str]) -> Dict[str, bool
     return labelers
 
 
-def check_folder_valid(base_dataset_dir: str):
-    try:
-        children_dirs = [os.path.basename(f.path.replace("\\", "/")) for f in os.scandir(base_dataset_dir) if f.is_dir()]
-        for children_dir in children_dirs:
-            if "Dataset" in children_dir:
-                return True
-        return False
-    except PermissionError:
-        return False
-
-
 def display_number_frames(num_frames: int):
     """
     Creates a sidebar display for the number of frames in the selected dataset
@@ -209,6 +176,12 @@ def preview_dataset(base_dataset_dir: str):
     if st.sidebar.button("Open Dataset"):
         folder_select()
 
+    if base_dataset_dir is None:
+        st.markdown("# Please open a dataset folder:")
+        if st.button("Open Dataset", key="second open dataset"):
+            folder_select()
+        return
+
     # Display name of dataset (Name of folder)
     dataset_name = os.path.abspath(base_dataset_dir).replace("\\", "/")
 
@@ -225,13 +198,10 @@ def preview_dataset(base_dataset_dir: str):
         # if it is not a datamaker dataset
         if instances is None:
             # Attempt to read as a normal perception dataset
-            if check_folder_valid(data_root):
-                ann_def, metric_def, cap = load_perception_dataset(data_root)
-            else:
-                ann_def = None
+            ds = Dataset(data_root)
+            if not ds.dataset_valid:
+                st.warning("The provided Dataset folder \"" + data_root + "\" is not considered valid")
 
-            # if it fails to open as a normal percpetion dataset then consider it to be an invalid folder
-            if ann_def is None:
                 st.markdown("# Please open a dataset folder:")
                 if st.button("Open Dataset", key="second open dataset"):
                     folder_select()
@@ -241,18 +211,18 @@ def preview_dataset(base_dataset_dir: str):
                 st.sidebar.markdown("# Current dataset:")
                 st.sidebar.write(folder_name)
 
-            display_number_frames(len(cap.captures.to_dict('records')))
+            display_number_frames(ds.length())
 
-            available_labelers = [a["name"] for a in ann_def.table.to_dict('records')]
+            available_labelers = ds.get_available_labelers()
             labelers = create_sidebar_labeler_menu(available_labelers)
 
             # zoom_image is negative if the application isn't in zoom mode
             index = int(st.session_state.zoom_image)
             if index >= 0:
-                zoom(index, 0, ann_def, metric_def, cap, data_root, labelers)
+                zoom(index, 0, ds, labelers)
             else:
                 num_rows = 5
-                grid_view(num_rows, ann_def, cap, data_root, labelers)
+                grid_view(num_rows, ds, labelers)
 
         # if it is a datamaker dataset
         else:
@@ -285,189 +255,6 @@ def preview_dataset(base_dataset_dir: str):
             folder_select()
 
 
-def get_instance_by_capture_idx(
-        instances: Dict[int, Tuple[AnnotationDefinitions, MetricDefinitions, Captures, int, str]], index: int) \
-        -> int:
-    """ Gets the instance in instances that contains the given capture index
-
-    :param instances: Dictionary of instances
-    :type instances: Dict[int, Tuple[AnnotationDefinitions, MetricDefinitions, Captures, int, str]]
-    :param index: Capture index that we want
-    :type index: int
-
-    :return: key in dictionary
-    :rtype: int
-    """
-    total = 0
-    keys = list(instances.keys())
-    keys.sort()
-    for key in keys:
-        total = total + instances[key][3]
-        if int(index) <= total - 1:
-            return key
-
-
-def get_dataset_length_with_instances(
-        instances: Dict[int, Tuple[AnnotationDefinitions, MetricDefinitions, Captures, int, str]],
-        until_instance: int = -1) -> \
-        int:
-    """ Gets the total length of a dataset that goes over multiple instances (aka. Datamaker dataset)
-    optionally you can specify until which instance you want to count 
-    (the order is based by the natural ordering of keys)
-
-    :param instances: Dictionary of instances
-    :type instances: Dict[int, Tuple[AnnotationDefinitions, MetricDefinitions, Captures, int, str]]
-    :param until_instance: optional, will stop counting when it reaches the specified instance (non-inclusive)
-    :type until_instance: int
-    :return: length
-    :rtype: int
-    """
-    total = 0
-    keys = list(instances.keys())
-    keys.sort()
-    for key in keys:
-        if 0 <= until_instance <= key:
-            break
-        total = total + instances[key][3]
-    return total
-
-
-def get_annotation_id(ann_def: AnnotationDefinitions, name: str) -> Optional[str]:
-    """ gets annotation definition id of the specified annotation
-
-    :param ann_def: Annotations in dataset
-    :type ann_def: AnnotationDefinitions
-    :param name: Name of the annotation we want the id of
-    :type name: str
-    :return: annotation definition id
-    :rtype: str
-    """
-    for idx, a in enumerate(ann_def.table.to_dict('records')):
-        if a["name"] == name:
-            return a["id"]
-    return None
-
-
-def get_annotation_index(ann_def: AnnotationDefinitions, name: str) -> int:
-    """ gets annotation definition index of the specified annotation
-
-    :param ann_def: Annotations in dataset
-    :type ann_def: AnnotationDefinitions
-    :param name: Name of the annotation we want the id of
-    :type name: str
-    :return: index
-    :rtype: int
-    """
-    for idx, a in enumerate(ann_def.table.to_dict('records')):
-        if a["name"] == name:
-            return idx
-    return -1
-
-
-def custom_compare_filenames(filenames):
-    for i in range(len(filenames)):
-        filenames[i] = int(os.path.basename(filenames[i])[4:-4])
-    return filenames
-
-
-def get_image_with_labelers(
-        index: int,
-        ann_def: AnnotationDefinitions,
-        cap: Captures,
-        data_root: str,
-        labelers_to_use: Dict[str, bool],
-        max_size: int = 500) -> Image:
-    """ Creates a PIL image of the capture at index that has all the labelers_to_use visualized
-
-    :param index: The index of the frame we want
-    :type index: int
-    :param ann_def: The Annotations for the dataset
-    :type ann_def: AnnotationsDefinitions
-    :param cap: The Captures for the dataset
-    :type cap: Captures
-    :param data_root: The root of the 
-    :type data_root: str
-    :param labelers_to_use: Dictionary containing keys for the name of every labeler available in the given dataset
-                            and the corresponding value is a boolean representing whether or not to display it
-    :type labelers_to_use: Dict[str, bool]
-    :param max_size: Optional (Default: 500), determines the maximum size of width and height of the created image
-                     Useful for optimizing. In the visualizer, if the images were full sized: the browser would take too
-                     much time to display them
-    :type max_size: int
-    :return: The image with the labelers
-    :rtype: PIL.Image
-    """
-    captures = cap.filter(def_id=ann_def.table.to_dict('records')[0]["id"])
-    captures = captures.sort_values(by='filename', key=custom_compare_filenames).reset_index(drop=True)
-    capture = captures.loc[index, "filename"]
-    filename = os.path.join(data_root, capture)
-    image = Image.open(filename)
-
-    if 'bounding box' in labelers_to_use and labelers_to_use['bounding box']:
-        bounding_box_definition_id = get_annotation_id(ann_def, 'bounding box')
-        bb_captures = cap.filter(def_id=bounding_box_definition_id)
-        bb_captures = bb_captures.sort_values(by='filename', key=custom_compare_filenames).reset_index(drop=True)
-        init_definition = ann_def.get_definition(bounding_box_definition_id)
-        label_mappings = {
-            m["label_id"]: m["label_name"] for m in init_definition["spec"]
-        }
-        image = v.draw_image_with_boxes(
-            image,
-            index,
-            bb_captures,
-            label_mappings,
-        )
-
-    if 'keypoints' in labelers_to_use and labelers_to_use['keypoints']:
-        keypoints_definition_id = get_annotation_id(ann_def, 'keypoints')
-        kp_captures = cap.filter(def_id=keypoints_definition_id)
-        kp_captures = kp_captures.sort_values(by='filename', key=custom_compare_filenames).reset_index(drop=True)
-        annotations = kp_captures.loc[index, "annotation.values"]
-        templates = ann_def.table.to_dict('records')[get_annotation_index(ann_def, 'keypoints')]['spec']
-        v.draw_image_with_keypoints(image, annotations, templates)
-
-    if 'bounding box 3D' in labelers_to_use and labelers_to_use['bounding box 3D']:
-        bounding_box_3d_definition_id = get_annotation_id(ann_def, 'bounding box 3D')
-        box_captures = cap.filter(def_id=bounding_box_3d_definition_id)
-        box_captures = box_captures.sort_values(by='filename', key=custom_compare_filenames).reset_index(drop=True)
-        annotations = box_captures.loc[index, "annotation.values"]
-        sensor = box_captures.loc[index, "sensor"]
-        image = v.draw_image_with_box_3d(image, sensor, annotations, None)
-
-    # bounding boxes and keypoints are depend on pixel coordinates so for now the thumbnail optimization applies only to
-    # segmentation
-    # TODO Make it so that bounding boxes and keypoints can be visualized at a lower resolution
-
-    image.thumbnail((max_size, max_size))
-    if 'semantic segmentation' in labelers_to_use and labelers_to_use['semantic segmentation']:
-        semantic_segmentation_definition_id = get_annotation_id(ann_def, 'semantic segmentation')
-
-        seg_captures = cap.filter(def_id=semantic_segmentation_definition_id)
-        seg_captures = seg_captures.sort_values(by='filename', key=custom_compare_filenames).reset_index(drop=True)
-        seg_filename = os.path.join(data_root, seg_captures.loc[index, "annotation.filename"])
-        seg = Image.open(seg_filename)
-        seg.thumbnail((max_size, max_size))
-
-        image = v.draw_image_with_segmentation(
-            image, seg
-        )
-
-    if 'instance segmentation' in labelers_to_use and labelers_to_use['instance segmentation']:
-        instance_segmentation_definition_id = get_annotation_id(ann_def, 'instance segmentation')
-
-        inst_captures = cap.filter(def_id=instance_segmentation_definition_id)
-        inst_captures = inst_captures.sort_values(by='filename', key=custom_compare_filenames).reset_index(drop=True)
-        inst_filename = os.path.join(data_root, inst_captures.loc[index, "annotation.filename"])
-        inst = Image.open(inst_filename)
-        inst.thumbnail((max_size, max_size))
-
-        image = v.draw_image_with_segmentation(
-            image, inst
-        )
-
-    return image
-
-
 def folder_select():
     """ Runs a subprocess that opens a file dialog to select a new directory, this will update st.session_state.curr_dir
     """
@@ -554,28 +341,25 @@ def create_grid_containers(num_rows: int, num_cols: int, start_at: int, dataset_
     return containers
 
 
-def grid_view(num_rows: int, ann_def: AnnotationDefinitions, cap: Captures, data_root: str, labelers: Dict[str, bool]):
+def grid_view(num_rows: int, ds: Dataset, labelers: Dict[str, bool]):
     """ Creates the grid view streamlit components
 
     :param num_rows: Number of rows
     :type num_rows: int
-    :param ann_def: Annotations for dataset
-    :type ann_def: AnnotationDefinitions
-    :param cap: Captures for dataset
-    :type cap: Captures
-    :param data_root: Path to dataset root
-    :type data_root: str
+    :param ds: Current Dataset
+    :type ds: Dataset
     :param labelers: Dictionary containing keys for the name of every labeler available in the given dataset
                      and the corresponding value is a boolean representing whether or not to display it
     :type labelers: Dict[str, bool]
     """
-    num_cols, start_at = create_grid_view_controls(num_rows, len(cap.captures.to_dict('records')))
+    dataset_size = ds.length()
 
-    containers = create_grid_containers(num_rows, num_cols, start_at, len(cap.captures.to_dict('records')))
+    num_cols, start_at = create_grid_view_controls(num_rows, dataset_size)
 
-    for i in range(start_at, min(start_at + (num_cols * num_rows), len(cap.captures.to_dict('records')))):
-        image = get_image_with_labelers(i, ann_def, cap, data_root, labelers,
-                                        max_size=get_resolution_from_num_cols(num_cols))
+    containers = create_grid_containers(num_rows, num_cols, start_at, dataset_size)
+
+    for i in range(start_at, min(start_at + (num_cols * num_rows), dataset_size)):
+        image = ds.get_image_with_labelers(i, labelers, max_size=get_resolution_from_num_cols(num_cols))
         containers[i - start_at].image(image, caption=str(i), use_column_width=True)
 
 
@@ -615,10 +399,7 @@ def grid_view_instances(
 
 def zoom(index: int,
          offset: int,
-         ann_def: AnnotationDefinitions,
-         metrics_def: MetricDefinitions,
-         cap: Captures,
-         data_root: str,
+         ds: Dataset,
          labelers: Dict[str, bool]):
     """ Creates streamlit components for Zoom in view
 
@@ -627,19 +408,13 @@ def zoom(index: int,
     :param offset: Is how much the index needs to be offset, this is only needed to 
                    handle multiple instances (Datamaker datasets)
     :type offset: int
-    :param ann_def: Annotations for Dataset
-    :type ann_def: AnnotationsDefinitions
-    :param metrics_def: Metrics for the dataset
-    :type metrics_def: MetricsDefinitions
-    :param cap: Captures for Dataset
-    :type cap: Captures
-    :param data_root: Path to dataset (Note that when using instances this is the path to the instance the index is in)
-    :type data_root: str
+    :param ds: Current Dataset
+    :type ds: Dataset
     :param labelers: Dictionary containing keys for the name of every labeler available in the given dataset
                      and the corresponding value is a boolean representing whether or not to display it
     :type labelers: Dict[str, bool]
     """
-    dataset_size = len(cap.captures.to_dict('records'))
+    dataset_size = ds.length()
 
     st.session_state.start_at = index
     st.session_state.zoom_image = index
@@ -664,18 +439,18 @@ def zoom(index: int,
     components.html("""<hr style="height:2px;border:none;color:#AAA;background-color:#AAA;" /> """, height=30)
 
     index = index - offset
-    image = get_image_with_labelers(index, ann_def, cap, data_root, labelers, max_size=2000)
+    image = ds.get_image_with_labelers(index, labelers, max_size=2000)
 
     st.image(image, use_column_width=True)
     layout = st.beta_columns(2)
     layout[0].title("Captures Metadata")
 
     captures_dir = None
-    for directory in os.walk(data_root):
+    for directory in os.walk(ds.data_root):
         name = str(directory[0]).replace('\\', '/').split('/')[-1]
         if name.startswith("Dataset") and \
                 "." not in name[1:] and \
-                os.path.abspath(data_root) != os.path.abspath(directory[0]):
+                os.path.abspath(ds.data_root) != os.path.abspath(directory[0]):
             captures_dir = os.path.abspath(directory[0])
             break
 
@@ -702,7 +477,7 @@ def zoom(index: int,
     with layout[1]:
         for metric in metrics:
             if metric['sequence_id'] == capture['sequence_id'] and metric['step'] == capture['step']:
-                for metric_def in metrics_def.table.to_dict('records'):
+                for metric_def in ds.get_metrics_records():
                     if metric_def['id'] == metric['metric_definition']:
                         st.markdown("## " + metric_def['name'])
                 st.write(metric)
@@ -732,4 +507,4 @@ if __name__ == "__main__":
     if os.path.isdir(args.data):
         preview_app({"data": args.data})
     else:
-        preview_app({"data": ""})
+        preview_app({"data": None})
